import os
import logging
from sklearn.metrics import mean_squared_error, r2_score
from transformers import BertForSequenceClassification, BertTokenizer
from datasets import load_dataset
from modelhub.clients import MLflowClient
from dotenv import load_dotenv

load_dotenv()

logger = logging.getLogger(__name__)
logger.setLevel(logging.DEBUG)

# Initialize the MLflow client
client = MLflowClient(base_url=os.getenv("MODELHUB_BASE_URL"))
experiment_id = os.getenv("MLFLOW_EXPERIMENT_ID")

client.set_experiment(experiment_id=experiment_id)

def evaluate_model():
    # Read the run_id from the file
    run_id = client.get_previous_stage_run_id()

    logger.info("Previous stage run_id: %s", run_id)
    
    # Load dataset
    dataset = load_dataset("imdb")
    test_dataset = dataset["test"].shuffle(seed=42).select(range(100))  # Select 100 samples for quick evaluation

    # Load tokenizer
    tokenizer = BertTokenizer.from_pretrained("bert-base-uncased")

    # Tokenize the dataset
    def tokenize_function(examples):
        return tokenizer(examples["text"], padding="max_length", truncation=True)

    test_dataset = test_dataset.map(tokenize_function, batched=True)
    test_dataset.set_format("torch", columns=["input_ids", "attention_mask", "label"])

    # Prepare test data
    X_test = test_dataset["input_ids"]
    y_test = test_dataset["label"]

    # Load the model using the run_id
    model_uri = f"runs:/{run_id}/model"
    model = client.load_model(model_uri)

    # Make predictions
    y_pred = model.predict(X_test)

    # Calculate metrics
    mse = mean_squared_error(y_test, y_pred)
    r2 = r2_score(y_test, y_pred)

    # Log metrics
    with client.start_run(run_name="evaluate") as run:
        client.log_metric("mse", mse)
        client.log_metric("r2", r2)

    print("Model evaluation complete")

if __name__ == "__main__":
    evaluate_model()


import os
import argparse
from dotenv import load_dotenv
from transformers import BertTokenizer, BertForSequenceClassification, Trainer, TrainingArguments
from datasets import load_dataset
from modelhub.clients import MLflowClient

load_dotenv()

# Initialize the MLflow client
client = MLflowClient(base_url=os.getenv("MODELHUB_BASE_URL"))
experiment_id = os.getenv("MLFLOW_EXPERIMENT_ID")

client.set_experiment(experiment_id=experiment_id)

def train_and_log_model(output_path: str):
    # Load dataset
    dataset = load_dataset("imdb")
    train_dataset = dataset["train"].shuffle(seed=42).select(range(100))  # Select 100 samples for quick training
    test_dataset = dataset["test"].shuffle(seed=42).select(range(100))    # Select 100 samples for quick testing

    # Load tokenizer and model
    tokenizer = BertTokenizer.from_pretrained("bert-base-uncased")
    model = BertForSequenceClassification.from_pretrained("bert-base-uncased")

    # Tokenize the dataset
    def tokenize_function(examples):
        return tokenizer(examples["text"], padding="max_length", truncation=True)

    train_dataset = train_dataset.map(tokenize_function, batched=True)
    test_dataset = test_dataset.map(tokenize_function, batched=True)
    
    train_dataset.set_format("torch", columns=["input_ids", "attention_mask", "label"])
    test_dataset.set_format("torch", columns=["input_ids", "attention_mask", "label"])

    # Define training arguments
    training_args = TrainingArguments(
        output_dir="./"+output_path,
        evaluation_strategy="epoch",
        per_device_train_batch_size=8,
        per_device_eval_batch_size=8,
        num_train_epochs=1,
        weight_decay=0.01,
    )

    # Initialize Trainer
    trainer = Trainer(
        model=model,
        args=training_args,
        train_dataset=train_dataset,
        eval_dataset=test_dataset,
    )

    # Start an MLflow run
    with client.start_run(run_name="train") as run:
        try:
            # Train the model
            trainer.train()

            # Log the model
            client.mlflow.transformers.log_model(trainer.model, "model")

            print("Model training and logging complete")

        except Exception as e:
            print(f"An error occurred: {e}")
            client.end_run(status="FAILED")
            raise e


if __name__ == "__main__":    
    parser = argparse.ArgumentParser(description="Training")
    parser.add_argument("--output_path", type=str, required=True, help="Path to save the output data")
    
    args = parser.parse_args()
    train_and_log_model(args.output_path)


""" This module contains the Pydantic models for the ModelHub API. """
from typing import List, Optional, Dict, Union
from pydantic import BaseModel


class ResourceRequests(BaseModel):
    """
    Represents the resource requests for a stage.
    """
    cpu: Union[str, int]
    memory: str


class ResourceLimits(BaseModel):
    """
    Represents the resource limits for a stage.
    """
    cpu: Optional[Union[str, int]] = None
    memory: Optional[str] = None
    gpu: Optional[Union[str, int]] = None


class Resources(BaseModel):
    """
    Represents the resources for a stage.
    """
    requests: Optional[ResourceRequests] = None
    limits: Optional[ResourceLimits] = None


class Toleration(BaseModel):
    """
    Represents a toleration for a stage.
    """
    key: str
    operator: str
    value: str
    effect: str


class BlobStorageConfig(BaseModel):
    """
    Represents the blob storage configuration for a stage.
    """
    container: str
    blob_url: str
    mount_path: str


class Stage(BaseModel):
    """
    Represents a stage in a pipeline.
    """
    name: str
    type: str
    params: Optional[dict] = None
    depends_on: Optional[List[str]] = []
    script: Optional[str] = None
    requirements: Optional[str] = None
    resources: Optional[Resources] = None
    tolerations: Optional[List[Toleration]] = None
    node_selector: Optional[Dict[str, str]] = None
    blob_storage_config: Optional[BlobStorageConfig] = None  # New field for blob storage config


class PipelineCreateRequest(BaseModel):
    """
    Represents a request to create a pipeline.
    """
    name: str
    description: Optional[str] = None
    experiment_id: str
    dataset_id: str
    image_tag: str
    stages: List[Stage]


class Pipeline(BaseModel):
    """
    Represents a pipeline.
    """
    pipeline_id: str
    name: str
    description: Optional[str] = None
    experiment_id: str
    dataset_id: str
    image_tag: str
    stages: List[Stage]
    status: Optional[str] = None
    workflows: Optional[List[dict]] = []


class SubmitPipelineRequest(BaseModel):
    """Submit pipeline request model"""
    modelhub_base_url: Optional[str] = None
    modelhub_client_id: Optional[str] = None
    modelhub_client_secret: Optional[str] = None